@@ -1,4 +1,5 @@
 """Python collections in a functional-programming style."""
+from functional_itertools.cattrs import CAttrs
 from functional_itertools.classes import CDict
 from functional_itertools.classes import CFrozenSet
 from functional_itertools.classes import CIterable
@@ -8,13 +9,14 @@ from functional_itertools.errors import EmptyIterableError
 from functional_itertools.errors import MultipleElementsError
 
 
-__version__ = "0.5.33"
-_ = {
-    CDict,
-    CFrozenSet,
-    CIterable,
-    CList,
-    CSet,
-    EmptyIterableError,
-    MultipleElementsError,
-}
+__version__ = "0.6.0"
+__all__ = [
+    "CAttrs",
+    "CDict",
+    "CFrozenSet",
+    "CIterable",
+    "CList",
+    "CSet",
+    "EmptyIterableError",
+    "MultipleElementsError",
+]
