@@ -110,34 +110,22 @@ if VERSION in {Version.py36, Version.py37}:
 elif VERSION is Version.py38:
 
     def _accumulate_citerable(
-        self: "CIterable[_T]",
-        func: Callable[[_T, _U], _U] = add,
-        *,
-        initial: Optional[_U] = None,
+        self: "CIterable[_T]", func: Callable[[_T, _U], _U] = add, *, initial: Optional[_U] = None,
     ) -> "CIterable[Union[_T, _U]]":
         return CIterable(accumulate(self._iterable, func, initial=initial))
 
     def _accumulate_clist(
-        self: "CList[_T]",
-        func: Callable[[_T, _U], _U] = add,
-        *,
-        initial: Optional[_U] = None,
+        self: "CList[_T]", func: Callable[[_T, _U], _U] = add, *, initial: Optional[_U] = None,
     ) -> "CList[Union[_T, _U]]":
         return self.iter().accumulate(func=func, initial=initial).list()
 
     def _accumulate_cset(
-        self: "CSet[_T]",
-        func: Callable[[_T, _T], _T] = add,
-        *,
-        initial: Optional[_U] = None,
+        self: "CSet[_T]", func: Callable[[_T, _T], _T] = add, *, initial: Optional[_U] = None,
     ) -> "CSet[_T]":
         return self.iter().accumulate(func=func, initial=initial).set()
 
     def _accumulate_cfrozenset(
-        self: "CFrozenSet[_T]",
-        func: Callable[[_T, _T], _T] = add,
-        *,
-        initial: Optional[_U] = None,
+        self: "CFrozenSet[_T]", func: Callable[[_T, _T], _T] = add, *, initial: Optional[_U] = None,
     ) -> "CFrozenSet[_T]":
         return self.iter().accumulate(func=func, initial=initial).frozenset()
 
@@ -154,17 +142,15 @@ class CIterable(Iterable[_T]):
             iter(iterable)
         except TypeError as error:
             (msg,) = error.args
-            raise TypeError(
-                f"{type(self).__name__} expected an iterable, but {msg}",
-            )
+            raise TypeError(f"{type(self).__name__} expected an iterable, but {msg}")
         else:
             self._iterable = iterable
 
-    @overload  # noqa: U100
+    @overload
     def __getitem__(self: "CIterable[_T]", item: int) -> _T:
         ...  # pragma: no cover
 
-    @overload  # noqa: F811,U100
+    @overload
     def __getitem__(self: "CIterable[_T]", item: slice) -> "CIterable[_T]":
         ...  # pragma: no cover
 
@@ -175,23 +161,17 @@ class CIterable(Iterable[_T]):
             if item < 0:
                 raise IndexError(f"Expected a non-negative index; got {item}")
             elif item > maxsize:
-                raise IndexError(
-                    f"Expected an index at most {maxsize}; got {item}",
-                )
+                raise IndexError(f"Expected an index at most {maxsize}; got {item}")
             else:
                 slice_ = islice(self._iterable, item, item + 1)
                 try:
                     return next(slice_)
                 except StopIteration:
-                    raise IndexError(
-                        f"{type(self).__name__} index out of range",
-                    )
+                    raise IndexError(f"{type(self).__name__} index out of range")
         elif isinstance(item, slice):
             return self.islice(item.start, item.stop, item.step)
         else:
-            raise TypeError(
-                f"Expected an int or slice; got a(n) {type(item).__name__}",
-            )
+            raise TypeError(f"Expected an int or slice; got a(n) {type(item).__name__}")
 
     def __iter__(self: "CIterable[_T]") -> Iterator[_T]:
         yield from self._iterable
@@ -213,14 +193,10 @@ class CIterable(Iterable[_T]):
     def dict(self: "CIterable[Tuple[_T,_U]]") -> "CDict[_T, _U]":
         return CDict(dict(self._iterable))
 
-    def enumerate(
-        self: "CIterable[_T]", start: int = 0,
-    ) -> "CIterable[Tuple[int, _T]]":
+    def enumerate(self: "CIterable[_T]", start: int = 0) -> "CIterable[Tuple[int, _T]]":
         return CIterable(enumerate(self._iterable, start=start))
 
-    def filter(
-        self: "CIterable[_T]", func: Optional[Callable[[_T], bool]],
-    ) -> "CIterable[_T]":
+    def filter(self: "CIterable[_T]", func: Optional[Callable[[_T], bool]]) -> "CIterable[_T]":
         return CIterable(filter(func, self._iterable))
 
     def frozenset(self: "CIterable[_T]") -> "CFrozenSet[_T]":
@@ -269,10 +245,7 @@ class CIterable(Iterable[_T]):
         return CSet(self._iterable)
 
     def sorted(
-        self: "CIterable[_T]",
-        *,
-        key: Optional[Callable[[_T], Any]] = None,
-        reverse: bool = False,
+        self: "CIterable[_T]", *, key: Optional[Callable[[_T], Any]] = None, reverse: bool = False,
     ) -> "CList[_T]":
         return CList(sorted(self._iterable, key=key, reverse=reverse))
 
@@ -283,9 +256,7 @@ class CIterable(Iterable[_T]):
     def tuple(self: "CIterable[_T]") -> Tuple[_T, ...]:
         return tuple(self._iterable)
 
-    def zip(
-        self: "CIterable[_T]", *iterables: Iterable[_U],
-    ) -> "CIterable[Tuple[Union[_T, _U]]]":
+    def zip(self: "CIterable[_T]", *iterables: Iterable[_U]) -> "CIterable[Tuple[Union[_T, _U]]]":
         return CIterable(zip(self._iterable, *iterables))
 
     # functools
@@ -312,9 +283,7 @@ class CIterable(Iterable[_T]):
     # itertools
 
     @classmethod
-    def count(
-        cls: Type["CIterable"], start: int = 0, step: int = 1,
-    ) -> "CIterable[int]":
+    def count(cls: Type["CIterable"], start: int = 0, step: int = 1) -> "CIterable[int]":
         return cls(count(start=start, step=step))
 
     def cycle(self: "CIterable[_T]") -> "CIterable[_T]":
@@ -322,39 +291,33 @@ class CIterable(Iterable[_T]):
 
     @classmethod
     def repeat(
-        cls: Type["CIterable[_T]"],
-        x: _T,
-        times: Union[int, Sentinel] = sentinel,
+        cls: Type["CIterable[_T]"], x: _T, times: Union[int, Sentinel] = sentinel,
     ) -> "CIterable[_T]":
         args, _ = drop_sentinel(times)
         return cls(repeat(x, *args))
 
     accumulate = _accumulate_citerable
 
-    def chain(
-        self: "CIterable[_T]", *iterables: Iterable[_U],
-    ) -> "CIterable[Union[_T, _U]]":
+    def chain(self: "CIterable[_T]", *iterables: Iterable[_U]) -> "CIterable[Union[_T, _U]]":
         return CIterable(chain(self._iterable, *iterables))
 
-    def compress(
-        self: "CIterable[_T]", selectors: Iterable[Any],
-    ) -> "CIterable[_T]":
+    def compress(self: "CIterable[_T]", selectors: Iterable[Any]) -> "CIterable[_T]":
         return CIterable(compress(self._iterable, selectors))
 
-    def dropwhile(
-        self: "CIterable[_T]", func: Callable[[_T], bool],
-    ) -> "CIterable[_T]":
+    def dropwhile(self: "CIterable[_T]", func: Callable[[_T], bool]) -> "CIterable[_T]":
         return CIterable(dropwhile(func, self._iterable))
 
-    def filterfalse(
-        self: "CIterable[_T]", func: Callable[[_T], bool],
-    ) -> "CIterable[_T]":
+    def filterfalse(self: "CIterable[_T]", func: Callable[[_T], bool]) -> "CIterable[_T]":
         return CIterable(filterfalse(func, self._iterable))
 
     def groupby(
         self: "CIterable[_T]", key: Optional[Callable[[_T], _U]] = None,
-    ) -> "CIterable[Tuple[_U, Iterator[_T]]]":
-        return CIterable(groupby(self._iterable, key=key))
+    ) -> "CIterable[Tuple[_U, CIterable[_T]]]":
+        def inner(x: Tuple[_U, Iterator[_T]]) -> Tuple[_U, CIterable[_T]]:
+            key, group = x
+            return key, CIterable(group)
+
+        return CIterable(groupby(self._iterable, key=key)).map(inner)
 
     def islice(
         self: "CIterable[_T]",
@@ -370,20 +333,16 @@ class CIterable(Iterable[_T]):
     ) -> "CIterable[_U]":
         return CIterable(starmap(func, self._iterable))
 
-    def takewhile(
-        self: "CIterable[_T]", func: Callable[[_T], bool],
-    ) -> "CIterable[_T]":
+    def takewhile(self: "CIterable[_T]", func: Callable[[_T], bool]) -> "CIterable[_T]":
         return CIterable(takewhile(func, self._iterable))
 
     def tee(self: "CIterable[_T]", n: int = 2) -> "CIterable[Iterator[_T]]":
-        return CIterable(tee(self._iterable, n))
+        return CIterable(tee(self._iterable, n)).map(CIterable)
 
     def zip_longest(
         self: "CIterable[_T]", *iterables: Iterable[_U], fillvalue: _V = None,
     ) -> "CIterable[Tuple[Union[_T, _U, _V]]]":
-        return CIterable(
-            zip_longest(self._iterable, *iterables, fillvalue=fillvalue),
-        )
+        return CIterable(zip_longest(self._iterable, *iterables, fillvalue=fillvalue))
 
     def product(
         self: "CIterable[_T]", *iterables: Iterable[_U], repeat: int = 1,
@@ -395,9 +354,7 @@ class CIterable(Iterable[_T]):
     ) -> "CIterable[Tuple[_T, ...]]":
         return CIterable(permutations(self._iterable, r=r))
 
-    def combinations(
-        self: "CIterable[_T]", r: int,
-    ) -> "CIterable[Tuple[_T, ...]]":
+    def combinations(self: "CIterable[_T]", r: int) -> "CIterable[Tuple[_T, ...]]":
         return CIterable(combinations(self._iterable, r))
 
     def combinations_with_replacement(
@@ -422,23 +379,17 @@ class CIterable(Iterable[_T]):
     def tail(self: "CIterable[_T]", n: int) -> "CIterable[_T]":
         return CIterable(tail(n, self._iterable))
 
-    def consume(
-        self: "CIterable[_T]", n: Optional[int] = None,
-    ) -> "CIterable[_T]":
+    def consume(self: "CIterable[_T]", n: Optional[int] = None) -> "CIterable[_T]":
         consume(self._iterable, n=n)
         return self
 
-    def nth(
-        self: "CIterable[_T]", n: int, default: _U = None,
-    ) -> Union[_T, _U]:
+    def nth(self: "CIterable[_T]", n: int, default: _U = None) -> Union[_T, _U]:
         return nth(self._iterable, n, default=default)
 
     def all_equal(self: "CIterable[Any]") -> bool:
         return all_equal(self._iterable)
 
-    def quantify(
-        self: "CIterable[_T]", pred: Callable[[_T], bool] = bool,
-    ) -> int:
+    def quantify(self: "CIterable[_T]", pred: Callable[[_T], bool] = bool) -> int:
         return quantify(self._iterable, pred=pred)
 
     def padnone(self: "CIterable[_T]") -> "CIterable[Optional[_T]]":
@@ -455,10 +406,7 @@ class CIterable(Iterable[_T]):
 
     @classmethod
     def repeatfunc(
-        cls: Type["CIterable"],
-        func: Callable[..., _T],
-        times: Optional[int] = None,
-        *args: Any,
+        cls: Type["CIterable"], func: Callable[..., _T], times: Optional[int] = None, *args: Any,
     ) -> "CIterable[_T]":
         return cls(repeatfunc(func, times=times, *args))
 
@@ -478,9 +426,7 @@ class CIterable(Iterable[_T]):
     def powerset(self: "CIterable[_T]") -> "CIterable[Tuple[_T,...]]":
         return CIterable(powerset(self._iterable))
 
-    def roundrobin(
-        self: "CIterable[_T]", *iterables: Iterable[_U],
-    ) -> "CIterable[Tuple[_T, _U]]":
+    def roundrobin(self: "CIterable[_T]", *iterables: Iterable[_U]) -> "CIterable[Tuple[_T, _U]]":
         return CIterable(roundrobin(self._iterable, *iterables))
 
     def unique_everseen(
@@ -503,9 +449,7 @@ class CIterable(Iterable[_T]):
         return cls(iter_except(func, exception, first=first))
 
     def first_true(
-        self: "CIterable[_T]",
-        default: _U = False,
-        pred: Optional[Callable[[_T], Any]] = None,
+        self: "CIterable[_T]", default: _U = False, pred: Optional[Callable[[_T], Any]] = None,
     ) -> Union[_T, _U]:
         return first_true(self._iterable, default=default, pred=pred)
 
@@ -522,23 +466,16 @@ class CIterable(Iterable[_T]):
     def random_combination(self: "CIterable[_T]", r: int) -> Tuple[_T, ...]:
         return random_combination(self._iterable, r)
 
-    def random_combination_with_replacement(
-        self: "CIterable[_T]", r: int,
-    ) -> Tuple[_T, ...]:
+    def random_combination_with_replacement(self: "CIterable[_T]", r: int) -> Tuple[_T, ...]:
         return random_combination_with_replacement(self._iterable, r)
 
-    def nth_combination(
-        self: "CIterable[_T]", r: int, index: int,
-    ) -> Tuple[_T, ...]:
+    def nth_combination(self: "CIterable[_T]", r: int, index: int) -> Tuple[_T, ...]:
         return nth_combination(self._iterable, r, index)
 
     # multiprocessing
 
     def pmap(
-        self: "CIterable[_T]",
-        func: Callable[[_T], _U],
-        *,
-        processes: Optional[int] = None,
+        self: "CIterable[_T]", func: Callable[[_T], _U], *, processes: Optional[int] = None,
     ) -> "CIterable[_U]":
         with Pool(processes=processes) as pool:
             return CIterable(pool.map(func, self._iterable))
@@ -546,9 +483,7 @@ class CIterable(Iterable[_T]):
     # pathlib
 
     @classmethod
-    def iterdir(
-        cls: Type["CIterable"], path: Union[Path, str],
-    ) -> "CIterable[Path]":
+    def iterdir(cls: Type["CIterable"], path: Union[Path, str]) -> "CIterable[Path]":
         return cls(Path(path).iterdir())
 
     # extra public
@@ -588,9 +523,7 @@ class CIterable(Iterable[_T]):
         index: int = 0,
         **kwargs: Any,
     ) -> "CIterable[_U]":
-        new_args = chain(
-            islice(args, index), [self._iterable], islice(args, index, None),
-        )
+        new_args = chain(islice(args, index), [self._iterable], islice(args, index, None))
         return CIterable(func(*new_args, **kwargs))
 
 
@@ -614,9 +547,7 @@ class CList(List[_T]):
     def enumerate(self: "CList[_T]", start: int = 0) -> "CList[Tuple[int, _T]]":
         return self.iter().enumerate(start=start).list()
 
-    def filter(
-        self: "CList[_T]", func: Optional[Callable[[_T], bool]],
-    ) -> "CList[_T]":
+    def filter(self: "CList[_T]", func: Optional[Callable[[_T], bool]]) -> "CList[_T]":
         return self.iter().filter(func).list()
 
     def frozenset(self: "CList[_T]") -> "CFrozenSet[_T]":
@@ -628,9 +559,7 @@ class CList(List[_T]):
     def list(self: "CFrozenSet[_T]") -> "CList[_T]":
         return self.iter().list()
 
-    def map(
-        self: "CList[_T]", func: Callable[..., _U], *iterables: Iterable,
-    ) -> "CList[_U]":
+    def map(self: "CList[_T]", func: Callable[..., _U], *iterables: Iterable) -> "CList[_U]":
         return self.iter().map(func, *iterables).list()
 
     def max(
@@ -665,19 +594,13 @@ class CList(List[_T]):
         return self.iter().set()
 
     def sort(
-        self: "CList[_T]",
-        *,
-        key: Optional[Callable[[_T], Any]] = None,
-        reverse: bool = False,
+        self: "CList[_T]", *, key: Optional[Callable[[_T], Any]] = None, reverse: bool = False,
     ) -> "CList[_T]":
         warn("Use the 'sorted' method instead of 'sort'")
         return self.sorted(key=key, reverse=reverse)
 
     def sorted(
-        self: "CList[_T]",
-        *,
-        key: Optional[Callable[[_T], Any]] = None,
-        reverse: bool = False,
+        self: "CList[_T]", *, key: Optional[Callable[[_T], Any]] = None, reverse: bool = False,
     ) -> "CList[_T]":
         return self.iter().sorted(key=key, reverse=reverse)
 
@@ -687,11 +610,20 @@ class CList(List[_T]):
     def tuple(self: "CList[_T]") -> Tuple[_T, ...]:
         return self.iter().tuple()
 
-    def zip(
-        self: "CList[_T]", *iterables: Iterable[_U],
-    ) -> "CList[Tuple[Union[_T, _U]]]":
+    def zip(self: "CList[_T]", *iterables: Iterable[_U]) -> "CList[Tuple[Union[_T, _U]]]":
         return self.iter().zip(*iterables).list()
 
+    # functools
+
+    def reduce(
+        self: "CList[_T]",
+        func: Callable[[_T, _T], _T],
+        initial: Union[_U, Sentinel] = sentinel,
+        *,
+        cast: bool = False,
+    ) -> Any:
+        return self.iter().reduce(func, initial=initial, cast=cast)
+
     # itertools
 
     @classmethod
@@ -700,46 +632,33 @@ class CList(List[_T]):
 
     accumulate = _accumulate_clist
 
-    def chain(
-        self: "CList[_T]", *iterables: Iterable[_U],
-    ) -> "CList[Union[_T, _U]]":
+    def chain(self: "CList[_T]", *iterables: Iterable[_U]) -> "CList[Union[_T, _U]]":
         return self.iter().chain(*iterables).list()
 
     def compress(self: "CList[_T]", selectors: Iterable[Any]) -> "CList[_T]":
         return self.iter().compress(selectors).list()
 
-    def dropwhile(
-        self: "CList[_T]", func: Callable[[_T], bool],
-    ) -> "CList[_T]":
+    def dropwhile(self: "CList[_T]", func: Callable[[_T], bool]) -> "CList[_T]":
         return self.iter().dropwhile(func).list()
 
-    def filterfalse(
-        self: "CList[_T]", func: Callable[[_T], bool],
-    ) -> "CList[_T]":
+    def filterfalse(self: "CList[_T]", func: Callable[[_T], bool]) -> "CList[_T]":
         return self.iter().filterfalse(func).list()
 
     def groupby(
         self: "CList[_T]", key: Optional[Callable[[_T], _U]] = None,
-    ) -> "CList[Tuple[_U, Iterator[_T]]]":
-        return (
-            self.iter()
-            .groupby(key=key)
-            .map(lambda x: (x[0], CList(x[1])))
-            .list()
-        )
+    ) -> "CList[Tuple[_U, CList[_T]]]":
+        return self.iter().groupby(key=key).map(lambda x: (x[0], CList(x[1]))).list()
 
     def starmap(
         self: "CList[Tuple[_T, ...]]", func: Callable[[Tuple[_T, ...]], _U],
     ) -> "CList[_U]":
         return self.iter().starmap(func).list()
 
-    def takewhile(
-        self: "CList[_T]", func: Callable[[_T], bool],
-    ) -> "CList[_T]":
+    def takewhile(self: "CList[_T]", func: Callable[[_T], bool]) -> "CList[_T]":
         return self.iter().takewhile(func).list()
 
-    def tee(self: "CList[_T]", n: int = 2) -> "CList[Iterator[_T]]":
-        return self.iter().tee(n=n).list()
+    def tee(self: "CList[_T]", n: int = 2) -> "CList[CList[_T]]":
+        return self.iter().tee(n=n).list().map(CList)
 
     def zip_longest(
         self: "CList[_T]", *iterables: List, fillvalue: Any = None,
@@ -751,17 +670,13 @@ class CList(List[_T]):
     ) -> "CList[Tuple[Union[_T, _U], ...]]":
         return self.iter().product(*iterables, repeat=repeat).list()
 
-    def permutations(
-        self: "CList[_T]", r: Optional[int] = None,
-    ) -> "CList[Tuple[_T, ...]]":
+    def permutations(self: "CList[_T]", r: Optional[int] = None) -> "CList[Tuple[_T, ...]]":
         return self.iter().permutations(r=r).list()
 
     def combinations(self: "CList[_T]", r: int) -> "CList[Tuple[_T, ...]]":
         return self.iter().combinations(r).list()
 
-    def combinations_with_replacement(
-        self: "CList[_T]", r: int,
-    ) -> "CList[Tuple[_T, ...]]":
+    def combinations_with_replacement(self: "CList[_T]", r: int) -> "CList[Tuple[_T, ...]]":
         return self.iter().combinations_with_replacement(r).list()
 
     # itertools-recipes
@@ -815,9 +730,7 @@ class CList(List[_T]):
     def powerset(self: "CList[_T]") -> "CList[Tuple[_T,...]]":
         return self.iter().powerset().list()
 
-    def roundrobin(
-        self: "CList[_T]", *iterables: Iterable[_U],
-    ) -> "CList[Tuple[_T, _U]]":
+    def roundrobin(self: "CList[_T]", *iterables: Iterable[_U]) -> "CList[Tuple[_T, _U]]":
         return self.iter().roundrobin(*iterables).list()
 
     def unique_everseen(
@@ -840,9 +753,7 @@ class CList(List[_T]):
         return CIterable.iter_except(func, exception, first=first).list()
 
     def first_true(
-        self: "CList[_T]",
-        default: _U = False,
-        pred: Optional[Callable[[_T], Any]] = None,
+        self: "CList[_T]", default: _U = False, pred: Optional[Callable[[_T], Any]] = None,
     ) -> Union[_T, _U]:
         return self.iter().first_true(default=default, pred=pred).list()
 
@@ -851,42 +762,22 @@ class CList(List[_T]):
     ) -> Tuple[Union[_T, _U], ...]:
         return self.iter().random_product(*iterables, repeat=repeat)
 
-    def random_permutation(
-        self: "CList[_T]", r: Optional[int] = None,
-    ) -> Tuple[_T, ...]:
+    def random_permutation(self: "CList[_T]", r: Optional[int] = None) -> Tuple[_T, ...]:
         return self.iter().random_permutation(r=r)
 
     def random_combination(self: "CList[_T]", r: int) -> Tuple[_T, ...]:
         return self.iter().random_combination(r)
 
-    def random_combination_with_replacement(
-        self: "CList[_T]", r: int,
-    ) -> Tuple[_T, ...]:
+    def random_combination_with_replacement(self: "CList[_T]", r: int) -> Tuple[_T, ...]:
         return self.iter().random_combination_with_replacement(r)
 
-    def nth_combination(
-        self: "CList[_T]", r: int, index: int,
-    ) -> Tuple[_T, ...]:
+    def nth_combination(self: "CList[_T]", r: int, index: int) -> Tuple[_T, ...]:
         return self.iter().nth_combination(r, index)
 
-    # functools
-
-    def reduce(
-        self: "CList[_T]",
-        func: Callable[[_T, _T], _T],
-        initial: Union[_U, Sentinel] = sentinel,
-        *,
-        cast: bool = False,
-    ) -> Any:
-        return self.iter().reduce(func, initial=initial, cast=cast)
-
     # multiprocessing
 
     def pmap(
-        self: "CList[_T]",
-        func: Callable[[_T], _U],
-        *,
-        processes: Optional[int] = None,
+        self: "CList[_T]", func: Callable[[_T], _U], *, processes: Optional[int] = None,
     ) -> "CList[_U]":
         return self.iter().pmap(func, processes=processes).list()
 
@@ -928,9 +819,7 @@ class CSet(Set[_T]):
     def enumerate(self: "CSet[_T]", start: int = 0) -> "CSet[Tuple[int, _T]]":
         return self.iter().enumerate(start=start).set()
 
-    def filter(
-        self: "CSet[_T]", func: Optional[Callable[[_T], bool]],
-    ) -> "CSet[_T]":
+    def filter(self: "CSet[_T]", func: Optional[Callable[[_T], bool]]) -> "CSet[_T]":
         return self.iter().filter(func).set()
 
     def frozenset(self: "CSet[_T]") -> "CFrozenSet[_T]":
@@ -942,9 +831,7 @@ class CSet(Set[_T]):
     def list(self: "CSet[_T]") -> "CList[_T]":
         return self.iter().list()
 
-    def map(
-        self: "CSet[_T]", func: Callable[..., _U], *iterables: Iterable,
-    ) -> "CSet[_U]":
+    def map(self: "CSet[_T]", func: Callable[..., _U], *iterables: Iterable) -> "CSet[_U]":
         return self.iter().map(func, *iterables).set()
 
     def max(
@@ -976,10 +863,7 @@ class CSet(Set[_T]):
         return self.iter().set()
 
     def sorted(
-        self: "CSet[_T]",
-        *,
-        key: Optional[Callable[[_T], Any]] = None,
-        reverse: bool = False,
+        self: "CSet[_T]", *, key: Optional[Callable[[_T], Any]] = None, reverse: bool = False,
     ) -> "CList[_T]":
         return self.iter().sorted(key=key, reverse=reverse)
 
@@ -989,9 +873,7 @@ class CSet(Set[_T]):
     def tuple(self: "CSet[_T]") -> Tuple[_T, ...]:
         return self.iter().tuple()
 
-    def zip(
-        self: "CSet[_T]", *iterables: Iterable[_U],
-    ) -> "CSet[Tuple[Union[_T, _U]]]":
+    def zip(self: "CSet[_T]", *iterables: Iterable[_U]) -> "CSet[Tuple[Union[_T, _U]]]":
         return self.iter().zip(*iterables).set()
 
     # set & frozenset methods
@@ -999,19 +881,13 @@ class CSet(Set[_T]):
     def union(self: "CSet[_T]", *others: Iterable[_U]) -> "CSet[Union[_T, _U]]":
         return CSet(super().union(*others))
 
-    def intersection(
-        self: "CSet[_T]", *others: Iterable[_U],
-    ) -> "CSet[Union[_T, _U]]":
+    def intersection(self: "CSet[_T]", *others: Iterable[_U]) -> "CSet[Union[_T, _U]]":
         return CSet(super().intersection(*others))
 
-    def difference(
-        self: "CSet[_T]", *others: Iterable[_U],
-    ) -> "CSet[Union[_T, _U]]":
+    def difference(self: "CSet[_T]", *others: Iterable[_U]) -> "CSet[Union[_T, _U]]":
         return CSet(super().difference(*others))
 
-    def symmetric_difference(
-        self: "CSet[_T]", other: Iterable[_U],
-    ) -> "CSet[Union[_T, _U]]":
+    def symmetric_difference(self: "CSet[_T]", other: Iterable[_U]) -> "CSet[Union[_T, _U]]":
         return CSet(super().symmetric_difference(other))
 
     def copy(self: "CSet[_T]") -> "CSet[_T]":
@@ -1023,19 +899,14 @@ class CSet(Set[_T]):
         raise RuntimeError("Use the 'union' method instead of 'update'")
 
     def intersection_update(self: "CSet[_T]") -> NoReturn:
-        raise RuntimeError(
-            "Use the 'intersection' method instead of 'intersection_update'",
-        )
+        raise RuntimeError("Use the 'intersection' method instead of 'intersection_update'")
 
     def difference_update(self: "CSet[_T]") -> NoReturn:
-        raise RuntimeError(
-            "Use the 'difference' method instead of 'difference_update'",
-        )
+        raise RuntimeError("Use the 'difference' method instead of 'difference_update'")
 
     def symmetric_difference_update(self: "CSet[_T]") -> NoReturn:
         raise RuntimeError(
-            "Use the 'symmetric_difference' method instead of "
-            "'symmetric_difference_update'",
+            "Use the 'symmetric_difference' method instead of " "'symmetric_difference_update'",
         )
 
     def add(self: "CSet[_T]", element: _T) -> "CSet[_T]":
@@ -1077,9 +948,7 @@ class CSet(Set[_T]):
 
     accumulate = _accumulate_cset
 
-    def chain(
-        self: "CSet[_T]", *iterables: Iterable[_U],
-    ) -> "CSet[Union[_T, _U]]":
+    def chain(self: "CSet[_T]", *iterables: Iterable[_U]) -> "CSet[Union[_T, _U]]":
         return self.iter().chain(*iterables).set()
 
     def compress(self: "CSet[_T]", selectors: Iterable[Any]) -> "CSet[_T]":
@@ -1088,18 +957,32 @@ class CSet(Set[_T]):
     def dropwhile(self: "CSet[_T]", func: Callable[[_T], bool]) -> "CSet[_T]":
         return self.iter().dropwhile(func).set()
 
-    def filterfalse(
-        self: "CSet[_T]", func: Callable[[_T], bool],
-    ) -> "CSet[_T]":
+    def filterfalse(self: "CSet[_T]", func: Callable[[_T], bool]) -> "CSet[_T]":
         return self.iter().filterfalse(func).set()
 
+    def groupby(
+        self: "CSet[_T]", key: Optional[Callable[[_T], _U]] = None,
+    ) -> "CSet[Tuple[_U, CFrozenSet[_T]]]":
+        return self.iter().groupby(key=key).map(lambda x: (x[0], CFrozenSet(x[1]))).set()
+
+    def starmap(self: "CSet[Tuple[_T, ...]]", func: Callable[[Tuple[_T, ...]], _U]) -> "CSet[_U]":
+        return self.iter().starmap(func).set()
+
+    def takewhile(self: "CSet[_T]", func: Callable[[_T], bool]) -> "CSet[_T]":
+        return self.iter().takewhile(func).set()
+
+    def tee(self: "CSet[_T]", n: int = 2) -> "CSet[CFrozenSet[_T]]":
+        return self.iter().tee(n=n).set().map(CFrozenSet)
+
+    # itertools - recipes
+
+    def take(self: "CSet[_T]", n: int) -> "CSet[_T]":
+        return self.iter().take(n).set()
+
     # multiprocessing
 
     def pmap(
-        self: "CSet[_T]",
-        func: Callable[[_T], _U],
-        *,
-        processes: Optional[int] = None,
+        self: "CSet[_T]", func: Callable[[_T], _U], *, processes: Optional[int] = None,
     ) -> "CSet[_U]":
         return self.iter().pmap(func, processes=processes).set()
 
@@ -1138,14 +1021,10 @@ class CFrozenSet(FrozenSet[_T]):
     def dict(self: "CFrozenSet[Tuple[_T, _U]]") -> "CDict[_T, _U]":
         return self.iter().dict()
 
-    def enumerate(
-        self: "CFrozenSet[_T]", start: int = 0,
-    ) -> "CFrozenSet[Tuple[int, _T]]":
+    def enumerate(self: "CFrozenSet[_T]", start: int = 0) -> "CFrozenSet[Tuple[int, _T]]":
         return self.iter().enumerate(start=start).frozenset()
 
-    def filter(
-        self: "CFrozenSet[_T]", func: Optional[Callable[[_T], bool]],
-    ) -> "CFrozenSet[_T]":
+    def filter(self: "CFrozenSet[_T]", func: Optional[Callable[[_T], bool]]) -> "CFrozenSet[_T]":
         return self.iter().filter(func).frozenset()
 
     def frozenset(self: "CFrozenSet[_T]") -> "CFrozenSet[_T]":
@@ -1191,16 +1070,11 @@ class CFrozenSet(FrozenSet[_T]):
         return self.iter().set()
 
     def sorted(
-        self: "CFrozenSet[_T]",
-        *,
-        key: Optional[Callable[[_T], Any]] = None,
-        reverse: bool = False,
+        self: "CFrozenSet[_T]", *, key: Optional[Callable[[_T], Any]] = None, reverse: bool = False,
     ) -> "CList[_T]":
         return self.iter().sorted(key=key, reverse=reverse)
 
-    def sum(
-        self: "CFrozenSet[_T]", start: Union[_T, int] = 0,
-    ) -> Union[_T, int]:
+    def sum(self: "CFrozenSet[_T]", start: Union[_T, int] = 0) -> Union[_T, int]:
         return self.iter().sum(start=start)
 
     def tuple(self: "CFrozenSet[_T]") -> Tuple[_T, ...]:
@@ -1213,19 +1087,13 @@ class CFrozenSet(FrozenSet[_T]):
 
     # set & frozenset methods
 
-    def union(
-        self: "CFrozenSet[_T]", *others: Iterable[_U],
-    ) -> "CFrozenSet[Union[_T, _U]]":
+    def union(self: "CFrozenSet[_T]", *others: Iterable[_U]) -> "CFrozenSet[Union[_T, _U]]":
         return CFrozenSet(super().union(*others))
 
-    def intersection(
-        self: "CFrozenSet[_T]", *others: Iterable[_U],
-    ) -> "CFrozenSet[Union[_T, _U]]":
+    def intersection(self: "CFrozenSet[_T]", *others: Iterable[_U]) -> "CFrozenSet[Union[_T, _U]]":
         return CFrozenSet(super().intersection(*others))
 
-    def difference(
-        self: "CFrozenSet[_T]", *others: Iterable[_U],
-    ) -> "CFrozenSet[Union[_T, _U]]":
+    def difference(self: "CFrozenSet[_T]", *others: Iterable[_U]) -> "CFrozenSet[Union[_T, _U]]":
         return CFrozenSet(super().difference(*others))
 
     def symmetric_difference(
@@ -1255,42 +1123,50 @@ class CFrozenSet(FrozenSet[_T]):
 
     accumulate = _accumulate_cfrozenset
 
-    def chain(
-        self: "CFrozenSet[_T]", *iterables: Iterable[_U],
-    ) -> "CFrozenSet[Union[_T, _U]]":
+    def chain(self: "CFrozenSet[_T]", *iterables: Iterable[_U]) -> "CFrozenSet[Union[_T, _U]]":
         return self.iter().chain(*iterables).frozenset()
 
-    def compress(
-        self: "CFrozenSet[_T]", selectors: Iterable[Any],
-    ) -> "CFrozenSet[_T]":
+    def compress(self: "CFrozenSet[_T]", selectors: Iterable[Any]) -> "CFrozenSet[_T]":
         return self.iter().compress(selectors).frozenset()
 
-    def dropwhile(
-        self: "CFrozenSet[_T]", func: Callable[[_T], bool],
-    ) -> "CFrozenSet[_T]":
+    def dropwhile(self: "CFrozenSet[_T]", func: Callable[[_T], bool]) -> "CFrozenSet[_T]":
         return self.iter().dropwhile(func).frozenset()
 
-    def filterfalse(
-        self: "CFrozenSet[_T]", func: Callable[[_T], bool],
-    ) -> "CFrozenSet[_T]":
+    def filterfalse(self: "CFrozenSet[_T]", func: Callable[[_T], bool]) -> "CFrozenSet[_T]":
         return self.iter().filterfalse(func).frozenset()
 
+    def groupby(
+        self: "CFrozenSet[_T]", key: Optional[Callable[[_T], _U]] = None,
+    ) -> "CFrozenSet[Tuple[_U, CFrozenSet[_T]]]":
+        return self.iter().groupby(key=key).map(lambda x: (x[0], CFrozenSet(x[1]))).frozenset()
+
+    def starmap(
+        self: "CFrozenSet[Tuple[_T, ...]]", func: Callable[[Tuple[_T, ...]], _U],
+    ) -> "CFrozenSet[_U]":
+        return self.iter().starmap(func).frozenset()
+
+    def takewhile(self: "CFrozenSet[_T]", func: Callable[[_T], bool]) -> "CFrozenSet[_T]":
+        return self.iter().takewhile(func).frozenset()
+
+    def tee(self: "CFrozenSet[_T]", n: int = 2) -> "CFrozenSet[CFrozenSet[_T]]":
+        return self.iter().tee(n=n).frozenset().map(CFrozenSet)
+
+    # itertools - recipes
+
+    def take(self: "CFrozenSet[_T]", n: int) -> "CFrozenSet[_T]":
+        return self.iter().take(n).frozenset()
+
     # multiprocessing
 
     def pmap(
-        self: "CFrozenSet[_T]",
-        func: Callable[[_T], _U],
-        *,
-        processes: Optional[int] = None,
+        self: "CFrozenSet[_T]", func: Callable[[_T], _U], *, processes: Optional[int] = None,
     ) -> "CFrozenSet[_U]":
         return self.iter().pmap(func, processes=processes).frozenset()
 
     # pathlib
 
     @classmethod
-    def iterdir(
-        cls: Type["CFrozenset"], path: Union[Path, str],
-    ) -> "CFrozenSet[Path]":
+    def iterdir(cls: Type["CFrozenset"], path: Union[Path, str]) -> "CFrozenSet[Path]":
         return cls(CIterable.iterdir(path))
 
     # extra public
@@ -1340,27 +1216,21 @@ class CDict(Dict[_T, _U]):
     def any_items(self: "CDict[Any, Any]") -> bool:
         return self.items().any()
 
-    def filter_keys(
-        self: "CDict[_T, _U]", func: Callable[[_T], bool],
-    ) -> "CDict[_T, _U]":
+    def filter_keys(self: "CDict[_T, _U]", func: Callable[[_T], bool]) -> "CDict[_T, _U]":
         def inner(item: Tuple[_T, _U]) -> bool:
             key, _ = item
             return func(key)
 
         return self.items().filter(inner).dict()
 
-    def filter_values(
-        self: "CDict[_T, _U]", func: Callable[[_U], bool],
-    ) -> "CDict[_T, _U]":
+    def filter_values(self: "CDict[_T, _U]", func: Callable[[_U], bool]) -> "CDict[_T, _U]":
         def inner(item: Tuple[_T, _U]) -> bool:
             _, value = item
             return func(value)
 
         return self.items().filter(inner).dict()
 
-    def filter_items(
-        self: "CDict[_T, _U]", func: Callable[[_T, _U], bool],
-    ) -> "CDict[_T, _U]":
+    def filter_items(self: "CDict[_T, _U]", func: Callable[[_T, _U], bool]) -> "CDict[_T, _U]":
         def inner(item: Tuple[_T, _U]) -> bool:
             key, value = item
             return func(key, value)
@@ -1385,18 +1255,14 @@ class CDict(Dict[_T, _U]):
     def list_items(self: "CDict[_T, _U]") -> "CList[Tuple[_T, _U]]":
         return self.items().list()
 
-    def map_keys(
-        self: "CDict[_T, _U]", func: Callable[[_T], _V],
-    ) -> "CDict[_V, _U]":
+    def map_keys(self: "CDict[_T, _U]", func: Callable[[_T], _V]) -> "CDict[_V, _U]":
         def inner(item: Tuple[_T, _U]) -> Tuple[_V, _U]:
             key, value = item
             return func(key), value
 
         return self.items().map(inner).dict()
 
-    def map_values(
-        self: "CDict[_T, _U]", func: Callable[[_U], _V],
-    ) -> "CDict[_T, _V]":
+    def map_values(self: "CDict[_T, _U]", func: Callable[[_U], _V]) -> "CDict[_T, _V]":
         def inner(item: Tuple[_T, _U]) -> Tuple[_T, _V]:
             key, value = item
             return key, func(value)
