@@ -45,9 +45,7 @@ def _get_version() -> Version:
     try:
         return mapping[minor]
     except KeyError:  # pragma: no cover
-        raise UnsupportVersionError(
-            f"Expected Python 3.6-3.8; got 3.{minor}",
-        ) from None
+        raise UnsupportVersionError(f"Expected Python 3.6-3.8; got 3.{minor}") from None
 
 
 VERSION = _get_version()
