@@ -35,6 +35,7 @@ from typing import Type
 from typing import Union
 
 from hypothesis import assume
+from hypothesis import example
 from hypothesis import given
 from hypothesis import settings
 from hypothesis.strategies import booleans
@@ -49,10 +50,12 @@ from hypothesis.strategies import lists
 from hypothesis.strategies import none
 from hypothesis.strategies import text
 from hypothesis.strategies import tuples
-from more_itertools import flatten
-from more_itertools import prepend
-from more_itertools import take
+from more_itertools.recipes import flatten
+from more_itertools.recipes import prepend
+from more_itertools.recipes import tabulate
+from more_itertools.recipes import take
 from pytest import mark
+from pytest import param
 from pytest import raises
 
 from functional_itertools.classes import CDict
@@ -67,8 +70,6 @@ from functional_itertools.utilities import Sentinel
 from functional_itertools.utilities import sentinel
 from functional_itertools.utilities import VERSION
 from functional_itertools.utilities import Version
-from tests.test_utilities import int_and_int_to_int_funcs
-from tests.test_utilities import int_to_bool_funcs
 from tests.test_utilities import is_even
 from tests.test_utilities import lists_or_sets
 from tests.test_utilities import small_ints
@@ -78,9 +79,7 @@ from tests.test_utilities import small_ints
 def test_init(x: Union[int, List[int]]) -> None:
     if isinstance(x, int):
         with raises(
-            TypeError,
-            match="CIterable expected an iterable, "
-            "but 'int' object is not iterable",
+            TypeError, match="CIterable expected an iterable, " "but 'int' object is not iterable",
         ):
             CIterable(x)  # type: ignore
     else:
@@ -88,6 +87,8 @@ def test_init(x: Union[int, List[int]]) -> None:
 
 
 @given(x=lists(integers()), index=integers() | floats())
+@example(x=[], index=-1)
+@example(x=[], index=maxsize + 1)
 def test_get_item(x: List[int], index: Union[int, float]) -> None:
     y = CIterable(x)
     if isinstance(index, int):
@@ -106,8 +107,7 @@ def test_get_item(x: List[int], index: Union[int, float]) -> None:
                 y[index]
         else:
             with raises(
-                IndexError,
-                match=f"Expected an index at most {maxsize}; got {index}",
+                IndexError, match=f"Expected an index at most {maxsize}; got {index}",
             ):
                 y[index]
     else:
@@ -236,8 +236,7 @@ def test_max_and_min(
         y = getattr(cls(x), func.__name__)(**key_kwargs, **default_kwargs)
     except ValueError:
         with raises(
-            ValueError,
-            match=escape(f"{func.__name__}() arg is an empty sequence"),
+            ValueError, match=escape(f"{func.__name__}() arg is an empty sequence"),
         ):
             func(x, **key_kwargs, **default_kwargs)
     else:
@@ -253,11 +252,7 @@ def test_max_and_min(
     step=small_ints | just(sentinel),
 )
 def test_range(
-    cls: Type,
-    data: DataObject,
-    start: int,
-    stop: Union[int, Sentinel],
-    step: Union[int, Sentinel],
+    cls: Type, data: DataObject, start: int, stop: Union[int, Sentinel], step: Union[int, Sentinel],
 ) -> None:
     if step is sentinel:
         assume(stop is not sentinel)
@@ -280,10 +275,7 @@ def test_set(cls: Type, data: DataObject) -> None:
 @mark.parametrize("cls", [CIterable, CList, CSet, CFrozenSet])
 @given(data=data(), key=none() | just(neg), reverse=booleans())
 def test_sorted(
-    cls: Type,
-    data: DataObject,
-    key: Optional[Callable[[int], int]],
-    reverse: bool,
+    cls: Type, data: DataObject, key: Optional[Callable[[int], int]], reverse: bool,
 ) -> None:
     x, _ = data.draw(lists_or_sets(cls, integers()))
     y = cls(x).sorted(key=key, reverse=reverse)
@@ -328,17 +320,14 @@ def test_zip(cls: Type, data: DataObject, iterables: List[List[int]]) -> None:
 
 @mark.parametrize("cls", [CIterable, CList, CSet, CFrozenSet])
 @given(data=data(), initial=integers() | just(sentinel))
-def test_reduce(
-    cls: Type, data: DataObject, initial: Union[int, Sentinel],
-) -> None:
+def test_reduce(cls: Type, data: DataObject, initial: Union[int, Sentinel]) -> None:
     x, cast = data.draw(lists_or_sets(cls, integers()))
     args, _ = drop_sentinel(initial)
     try:
         y = cls(x).reduce(max, initial=initial)
     except EmptyIterableError:
         with raises(
-            TypeError,
-            match=escape("reduce() of empty sequence with no initial value"),
+            TypeError, match=escape("reduce() of empty sequence with no initial value"),
         ):
             reduce(max, x, *args)
     else:
@@ -357,18 +346,14 @@ def test_reduce_propagating_type_error(x: Tuple[int, int]) -> None:
 
 @mark.xfail
 @given(
-    x=lists(lists(integers())),
-    initial=lists(integers()) | just(sentinel),
-    n=small_ints,
+    x=lists(lists(integers())), initial=lists(integers()) | just(sentinel), n=small_ints,
 )
 def test_reduce_as_iterable(
     x: List[List[int]], initial: Union[List[int], Sentinel], n: int,
 ) -> None:
     args, _ = drop_sentinel(initial)
     try:
-        y = CIterable(x).reduce_as_iterable(
-            lambda x, y: list(chain(x, y)), initial=initial,
-        )
+        y = CIterable(x).reduce_as_iterable(lambda x, y: list(chain(x, y)), initial=initial)
     except EmptyIterableError:
         assume(False)
     else:
@@ -423,9 +408,7 @@ def test_repeat(cls: Type, data: DataObject, x: int) -> None:
     if VERSION in {Version.py36, Version.py37}
     else fixed_dictionaries({"initial": none() | integers()}),
 )
-def test_accumulate(
-    cls: Type, data: DataObject, initial: Dict[str, Any],
-) -> None:
+def test_accumulate(cls: Type, data: DataObject, initial: Dict[str, Any]) -> None:
     x, cast = data.draw(lists_or_sets(cls, integers()))
     y = cls(x).accumulate(max, **initial)
     assert isinstance(y, cls)
@@ -473,16 +456,23 @@ def test_filterfalse(cls: Type, data: DataObject) -> None:
         assert cast(y) == cast(filterfalse(is_even, x))
 
 
-@given(x=lists(integers()), key=none() | just(neg))
-def test_groupby(x: List[int], key: Optional[Callable[[int], int]]) -> None:
-    y = CIterable(x).groupby(key=key)
-    assert isinstance(y, CIterable)
-    y = list(y)
-    z = list(groupby(x, key=key))
+@mark.parametrize(
+    "cls", [CIterable, param(CList, marks=mark.xfail), CSet, CFrozenSet],
+)
+@given(data=data(), key=none() | just(neg))
+def test_groupby(cls: Type, data: DataObject, key: Optional[Callable[[int], int]]) -> None:
+    x, cast = data.draw(lists_or_sets(cls, integers()))
+    y = cls(x).groupby(key=key)
+    assert isinstance(y, cls)
+    y = cast(y)
+    z = cast(groupby(x, key=key))
     assert len(y) == len(z)
-    for (key_y, group_y), (key_z, group_z) in zip(y, z):
-        assert key_y == key_z
-        assert list(group_y) == list(group_z)
+    if cls in {CIterable, CList}:
+        for (key_y, group_y), (key_z, group_z) in zip(y, z):
+            assert key_y == key_z
+            assert isinstance(key_y, int)
+            assert isinstance(group_y, cls)
+            assert list(group_y) == list(group_z)
 
 
 @given(
@@ -492,10 +482,7 @@ def test_groupby(x: List[int], key: Optional[Callable[[int], int]]) -> None:
     step=small_ints | just(sentinel),
 )
 def test_islice(
-    x: List[int],
-    start: int,
-    stop: Union[int, Sentinel],
-    step: Union[int, Sentinel],
+    x: List[int], start: int, stop: Union[int, Sentinel], step: Union[int, Sentinel],
 ) -> None:
     if step is sentinel:
         assume(stop is not sentinel)
@@ -507,31 +494,41 @@ def test_islice(
     assert list(y) == list(islice(x, start, *args))
 
 
-@given(x=lists(tuples(integers(), integers())), func=int_and_int_to_int_funcs)
-def test_starmap(
-    x: List[Tuple[int, int]], func: Callable[[Tuple[int, int]], int],
-) -> None:
-    y = CIterable(x).starmap(func)
-    assert isinstance(y, CIterable)
-    assert list(y) == list(starmap(func, x))
+@mark.parametrize("cls", [CIterable, CList, CSet, CFrozenSet])
+@given(data=data())
+def test_starmap(cls: Type, data: DataObject) -> None:
+    x, cast = data.draw(lists_or_sets(cls, tuples(integers(), integers())))
+    y = cls(x).starmap(max)
+    assert isinstance(y, cls)
+    assert cast(y) == cast(starmap(max, x))
 
 
-@given(x=lists(integers()), func=int_to_bool_funcs)
-def test_takewhile(x: List[int], func: Callable[[int], bool]) -> None:
-    y = CIterable(x).takewhile(func)
-    assert isinstance(y, CIterable)
-    assert list(y) == list(takewhile(func, x))
+@mark.parametrize("cls", [CIterable, CList, CSet, CFrozenSet])
+@given(data=data())
+def test_takewhile(cls: Type, data: DataObject) -> None:
+    x, cast = data.draw(lists_or_sets(cls, integers()))
+    y = cls(x).takewhile(is_even)
+    assert isinstance(y, cls)
+    if cls in {CIterable, CList}:
+        assert cast(y) == cast(takewhile(is_even, x))
 
 
-@given(x=lists(integers()), n=small_ints)
-def test_tee(x: List[int], n: int) -> None:
-    y = CIterable(x).tee(n=n)
-    assert isinstance(y, CIterable)
-    y = list(y)
-    z = list(tee(x, n))
-    assert len(y) == len(z)
+@mark.parametrize("cls", [CIterable, CList, CSet, CFrozenSet])
+@given(data=data(), n=small_ints)
+def test_tee(cls: Type, data: DataObject, n: int) -> None:
+    x, cast = data.draw(lists_or_sets(cls, integers()))
+    y = cls(x).tee(n=n)
+    assert isinstance(y, cls)
+    y = cast(y)
+    z = cast(tee(x, n))
+    if cls in {CIterable, CList}:
+        assert len(y) == len(z)
     for y_i, z_i in zip(y, z):
-        assert list(y_i) == list(z_i)
+        if cls is CSet:
+            assert isinstance(y_i, CFrozenSet)
+        else:
+            assert isinstance(y_i, cls)
+        assert cast(y_i) == cast(z_i)
 
 
 @given(
@@ -545,9 +542,7 @@ def test_zip_longest(
 ) -> None:
     y = CIterable(x).zip_longest(*iterables, fillvalue=fillvalue)
     assert isinstance(y, CIterable)
-    assert list(y[:n]) == list(
-        islice(zip_longest(x, *iterables, fillvalue=fillvalue), n),
-    )
+    assert list(y[:n]) == list(islice(zip_longest(x, *iterables, fillvalue=fillvalue), n))
 
 
 @given(
@@ -585,11 +580,14 @@ def test_combinations_with_replacement(x: List[int], r: int, n: int) -> None:
 # itertools-recipes
 
 
-@given(x=lists(integers()), n=integers(0, maxsize))
-def test_take(x: List[int], n: int) -> None:
-    y = CIterable(x).take(n)
-    assert isinstance(y, CIterable)
-    assert list(y) == list(take(n, x))
+@mark.parametrize("cls", [CIterable, CList, CSet, CFrozenSet])
+@given(data=data(), n=integers(0, maxsize))
+def test_take(cls: Type, data: DataObject, n: int) -> None:
+    x, cast = data.draw(lists_or_sets(cls, integers()))
+    y = cls(x).take(n)
+    assert isinstance(y, cls)
+    if cls in {CIterable, CList}:
+        assert cast(y) == cast(take(n, x))
 
 
 @given(x=lists(integers()), value=integers())
@@ -599,6 +597,13 @@ def test_prepend(x: List[int], value: int) -> None:
     assert list(y) == list(prepend(value, x))
 
 
+@given(start=integers(), n=small_ints)
+def test_tabulate(start: int, n: int) -> None:
+    x = CIterable.tabulate(neg, start=start)
+    assert isinstance(x, CIterable)
+    assert list(islice(x, n)) == list(islice(tabulate(neg, start=start), n))
+
+
 # multiprocessing
 
 
@@ -619,9 +624,7 @@ def test_pmap(cls: Type, data: DataObject) -> None:
 @given(data=data(), use_path=booleans())
 def test_iterdir(cls: Type, data: DataObject, use_path: bool) -> None:
     x, func = data.draw(
-        lists_or_sets(
-            cls, text(alphabet=ascii_lowercase, min_size=1), unique=True,
-        ),
+        lists_or_sets(cls, text(alphabet=ascii_lowercase, min_size=1), unique=True),
     )
     with TemporaryDirectory() as temp_dir_str:
         temp_dir = Path(temp_dir_str)
