@@ -32,9 +32,7 @@ def test_reversed(x: List[int]) -> None:
 
 
 @given(x=lists(integers()), key=none() | int_to_int_funcs, reverse=booleans())
-def test_sort(
-    x: List[int], key: Optional[Callable[[int], int]], reverse: bool,
-) -> None:
+def test_sort(x: List[int], key: Optional[Callable[[int], int]], reverse: bool) -> None:
     with warns(UserWarning, match="Use the 'sorted' method instead of 'sort'"):
         y = CList(x).sort(key=key, reverse=reverse)
     assert isinstance(y, CList)
